#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#clasesbase: Todas las clases base que no poseen fichero propio

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from Driza import salida

class Operacion:
    """Clase base de las operaciones. Cada una reimplementa el procedimiento según lo necesita"""
    def __init__(self,nombre,datos,config,widget,funcion,interfazresultado,initresultado,comprobarentrada):
        self.dato=datos
        self.nombre=nombre
        self.interfazresultado=interfazresultado
        self.initresultado=initresultado
        self.resultado=None
        self.funcion=funcion
        self.comprobarentrada=comprobarentrada
        from Driza.textrender import FloatRender
        self.fr=FloatRender(config)
        from Driza.widgets import woperaciones
        if widget["tipo"]=="CalculoSimple":
            self.widget=woperaciones.WidgetCalculoSimple(self.nombre,widget["opciones"],datos)
        elif widget["tipo"]=="BondadDeAjuste":
            self.widget=woperaciones.WidgetCalculoSimpleBondadAjuste(self.nombre,widget["listaoperaciones"],datos)
        elif widget["tipo"]=="CalculoDoble":
            self.widget=woperaciones.WidgetCalculoDoble(self.nombre,widget["opciones"],datos)
        elif widget["tipo"]=="ProporcionSimple":
            self.widget=woperaciones.WidgetCaso(self.nombre,widget["opciones"],datos)
        elif widget["tipo"]=="Graficos":
            self.widget=woperaciones.WidgetGraficos(self.nombre,widget["graficos"],widget["opciones"],datos)
        elif widget["tipo"]=="Distribucion":
            self.widget=woperaciones.WidgetDistribucion(self.nombre,widget["opcionescomun"],[],datos)
        elif widget["tipo"]=="2Variables3Casos":
            self.widget=woperaciones.Widget2Variable3Casos(self.nombre,widget["opciones"],datos)
        else:
            raise NameError

class OperacionCalculo(Operacion):
    """Los calculos que se realizan sobre un conjunto de variables por separado"""
    def procedimiento(self):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        variables=self.widget.seleccion()
        opciones=self.widget.opciones()
        opciones["caso"]=self.dato.agrupacion() #Añade esta opcion para saber si tiene que dibujar el caso o no
        self.comprobarentrada(opciones)

        lista=[]
        for variable in variables:
            if self.dato.agrupacion():
                for caso in self.dato.getCasos():
                    lista.append([variable,caso,self.funcion(self.dato,variable,caso,opciones)])
            else:
            #FIXME El caso puede ser cambiado a su representacion segun el diccionario de casos de la variable
                lista.append([variable,None,self.funcion(self.dato,variable,None,opciones)])
        
        #Creamos el resultado de salida
        resultado=salida.Resultado()
        self.initresultado(resultado,opciones)

        #Rellenamos con los resultados obtenidos por R

        for x in lista:
            self.interfazresultado(resultado,x,floatrender=self.fr)

        #Devolvemos el objeto
        return resultado

class OperacionDistribucion(Operacion):
    """Los calculos de modelos de distribuciones, no interviene variable"""
    def procedimiento(self):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        opciones=self.widget.opciones()
        self.comprobarentrada(opciones)

        salidar=self.funcion(self.dato,opciones)
        
        #Creamos el resultado de salida
        resultado=salida.Resultado()
        self.initresultado(resultado,opciones)

        #Rellenamos con los resultados obtenidos por R
        self.interfazresultado(resultado,salidar,floatrender=self.fr)

        #Devolvemos el objeto
        return resultado


class OperacionGraficos(Operacion):
    """Calculos de gráficas. Son diferentes a los calculos normales ya que la información que preguntan es acerca de los ejes. Además, solo se hace una gráfica por vez"""
    def procedimiento(self):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        opciones=self.widget.opciones()
        ejes=self.widget.ejes()
        self.comprobarentrada(opciones)
        salidar=self.funcion(self.dato,ejes,opciones)
        
        #Creamos el resultado de salida
        resultado=salida.Resultado()
        self.initresultado(resultado,opciones)

        #Rellenamos con los resultados obtenidos por R

        self.interfazresultado(resultado,ejes,salidar,floatrender=self.fr)

        #Devolvemos el objeto
        return resultado

class OperacionTabla(Operacion):
    """La diferencia respecto a la operacion de calculo es que initresultado requiere conocer los datos"""
    def procedimiento(self):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        variables=self.widget.seleccion()  #TODO: que Solo deje escoger una variable(Como gráficos)
        opciones=self.widget.opciones()
        self.comprobarentrada(opciones)
        opciones["caso"]=self.dato.agrupacion() #Añade esta opcion para saber si tiene que dibujar el caso o no

        lista=[]
        variable=variables[0]
        if self.dato.agrupacion():
            for caso in self.dato.getCasos():
                lista.append([variable,caso,self.funcion(self.dato,variable,caso,opciones)])
        else:
        #FIXME El caso puede ser cambiado a su representacion segun el diccionario de casos de la variable
            lista.append([variable,None,self.funcion(self.dato,variable,None,opciones)])
        
        #Creamos el resultado de salida
        resultado = salida.Resultado()
        self.initresultado(resultado,opciones,self.dato,variable)

        #Rellenamos con los resultados obtenidos por R

        for x in lista:
            self.interfazresultado(resultado,x,floatrender=self.fr)

        #Devolvemos el objeto
        return resultado


class OperacionMultiple(Operacion):
    """Esta operacion trabaja con todas las variables simultaneamente"""
    def procedimiento(self):
        """Esta funcion llama a la funcion "funcion" definida por cada operacion, y despues crea un objeto del tipo salida, tambien definido en cada operacion
        Finalmente, lo devuelve para la ventana de salida"""

        #Obtenemos los datos que ha introducido el usuario
        lasvariables=self.widget.seleccion() 
        opciones=self.widget.opciones()
        opciones["caso"]=self.dato.agrupacion() #Añade esta opcion para saber si tiene que dibujar el caso o no
        self.comprobarentrada(opciones)
        lista=[]
        variables=lasvariables
        if self.dato.agrupacion():
            for caso in self.dato.getCasos():
                lista.append([variables,caso,self.funcion(self.dato,variables,caso,opciones)])
        else:
        #FIXME El caso puede ser cambiado a su representacion segun el diccionario de casos de la variable
            lista.append([variables,None,self.funcion(self.dato,variables,None,opciones)])
        
        #Creamos el resultado de salida
        resultado=salida.Resultado()
        self.initresultado(resultado,opciones)

        #Rellenamos con los resultados obtenidos por R

        for x in lista:
            self.interfazresultado(resultado,x,floatrender=self.fr)

        #Devolvemos el objeto
        return resultado





class ElementoWidgetOperacion:
    """Es padre de cualquier elemento que pertenezca a un widget de operaciones
       Actualmente heredan SelectorVariableEstudio y WOpciones
    """
    pass




class Funcion:
    """Clase padre de todas las funciones del dialogo de creacion de variables"""
    def __init__(self):
        """Todas las clases hijas deben reescribir el valor de estos atributos"""
        self.nombre=""
        self.descripcion=""

    def __call__(self):
        """Todas las clases hijas deben reimplementar este método"""
        pass



class ListaFicheros(list):
    """
    Contenedor de la lista de ficheros. Funciona como cola LIFO, y hereda de list.
    El parametro pasado es la longitud de lista que se desea
    """
    def __init__(self,n,lista=None):
        #VARIABLES PRIVADAS
        self.__nelementos=n
        if isinstance(lista,list):
            list.__init__(self,lista)
        else:
            list.__init__(self)

    #FUNCIONES PUBLICAS
    def insert(self,elemento):
        """Inserta un elemento, garantizando que siempre se va a colocar en el primer lugar"""
        if elemento in self:
            self.remove(elemento)
        list.insert(self,0,elemento)
        if len(self)>self.__nelementos:
            self.pop()

    # Elementos a configurar:

    #tmpdir: Guarda la ruta del directorio de trabajo
    #vsplash: boolean que dice si aparece la ventana splash o no
    #lfichero: Lista de ficheros
    #version es un entero que representa la version del formato de configuración
class Configuracion(dict):
    def __init__(self):
        dict.__init__(self)
        listaficheros=ListaFicheros(5)
        self.__setitem__("version",2)
        self.__setitem__("tmpdir","/tmp")
        self.__setitem__("lfichero",listaficheros)
        self.__setitem__("decimales",3)
        self.__setitem__("nundo",5)
        self.__setitem__("vsplash",True)

    def creaDesdeTexto(self,diccionario):
        self.__setitem__("version",diccionario["version"])
        self.__setitem__("tmpdir",diccionario["tmpdir"])
        self.__setitem__("lfichero",ListaFicheros(5,diccionario["lfichero"].split(',')))
        self.__setitem__("decimales",eval(diccionario["decimales"]))
        self.__setitem__("nundo",eval(diccionario["nundo"]))
        self.__setitem__("vsplash",eval(diccionario['vsplash']))
