#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Fichero manejador de ficheros
#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""Módulo de interfaces"""

from rpy import r
from ConfigParser import ConfigParser

class InterfazFichero:
    """ Interfaz entre datos y ficheros. 
    Permite cargar o guardar un objeto contenedorregvar en un fichero. 
    Se comunica con una interfazdatos. 
    Es llamado por el gestor de ficheros """
    #FIXME: Esta clase no esta actuando como una interfaz. Los metodos \
    #save y load trabajan con la interfaz de datos directamente, actuando \
    #como gestora insertando el estado. En el caso de la importación de texto,\
    # tambien insertan el estado, aunque previamente lo crea al no poder \
    #cargarlo directamente con pickle.

    def __init__(self, interfazdatos):
        """
        Inicializa el manejador de ficheros.
        Crea los objetos que se usaran para comprobar la extension de los archivos
        """
        #VARIABLES PRIVADAS
        import re
        self.__id = interfazdatos
        #FIXME: El control de extensiones lo debe llevar el GestorFicheros, \
        #para asi advertir al usuario de lo que crea conveniente
        self.__driza = re.compile('.*\.driza$')
        self.__spss = re.compile('.*\.dat$')
        self.__txt = re.compile('.*\.txt$')

    #FUNCIONES PUBLICAS
    def save(self, fichero, datos, tipo = "Auto", sobreescribir = False):
        """
        Guarda un fichero en el formato standard (Pickle) 
        o en el formato indicado
        """
        import os
        if not sobreescribir and os.path.exists(fichero): #El fichero existe
            from Driza.excepciones import FicheroExisteException
            raise FicheroExisteException(fichero)
        archivo = open(fichero, 'w')
        if (tipo == "Auto" and self.__driza.match(fichero)) or tipo == "Driza":
            import pickle
            pickle.dump(datos, archivo)
        archivo.close()

    def load(self, fichero, tipo = "Auto", opciones = None):
        """
        Carga un fichero con el tipo pasado o autodeteccion de tipos 
        si no pasa ninguno
        """
        import os
        if not os.path.exists(fichero):
            from Driza.excepciones import FicheroNoExisteException
            raise FicheroNoExisteException(fichero)
        archivo = open(fichero, 'r')
        if (tipo == "Auto" and self.__driza.match(fichero)) or tipo == "Driza":
            import pickle
            datos = pickle.load(archivo) 
            archivo.close()
            self.__id.insertarestado(datos, flagoriginal=True)
        elif (tipo =="Auto" and self.__txt.match(fichero)) or tipo == "Texto":
            self.__importartexto(fichero, opciones["delimitadoratrib"], \
                    opciones["cabeceras"])
        elif self.__spss.match(fichero):
            pass

    #FUNCIONES PRIVADAS

    def __importartexto(self, fichero, delimitadoratrib, cabeceras):
        """Importa los datos de un fichero de texto"""
        if self.__txt.match(fichero):
            self.__id.borrartodo()
            datos = r.read_table(fichero, sep = delimitadoratrib, \
                    header = cabeceras, na_strings="NA", dec=".", \
                    strip_white = True)
            for pareja in datos.items():
                try:
                    float(pareja[1][0])
                except:
                    #Es un Factor
                    self.__id.appVar(pareja[0],"Factor")
                    self.__id[self.__id.getVar(pareja[0])] = pareja[1]
                else:
                    self.__id.appVar(pareja[0],"Real")
                    self.__id[self.__id.getVar(pareja[0])] = pareja[1]


class InterfazConfig:
    """
    Clase encargada de actuar como interfaz de la configuración. Permite 
    leer y guardar la configuración en ficheros.
    """
    def load(self, fichero):
        """
        Carga la configuración del fichero de configuración del programa, 
        devolviendo la configuración por defecto si no la encuentra. 
        El parametro fichero es opcional e indica un fichero alternativo 
        al de configuracion por defecto 
        """
        configfile = ConfigParser()
        from Driza.clasesbase import Configuracion
        config = Configuracion()
        archivo = open(fichero,"r")
        configfile.readfp(archivo)
        config.creaDesdeTexto(configfile._sections["General"])
        if config["version"]<2:
            from Driza.excepciones import VersionAnterior
            raise VersionAnterior(configuracion)
        archivo.close()
        return config


    def save(self, config, fichero):
        """
        Guarda la configuracion en el fichero de configuración del programa. 
        El parametro fichero es opcional e indica un fichero alternativo al 
        de configuracion por defecto
        """
        configfile = ConfigParser()
        config["lfichero"] = ",".join(config["lfichero"])
        configfile.add_section("General")

        for oper, valor in config.items():
            configfile.set("General", oper, valor)

        archivo = open(fichero, "w")
        configfile.write(archivo)
        archivo.close()

        #Volvemos a poner a un tipo ListaFicheros
        from Driza.clasesbase import ListaFicheros
        config["lfichero"] = ListaFicheros(5, config["lfichero"].split(","))

