#!/usr/bin/python
# -*- coding: utf-8 -*-

#widgetsqt.py: Todas aquellas clases que heredan directamente de algun widget de Qt y que pretenden aumentar su funcionalidad

#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez
#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#Fichero de libreria de qt
from qt import *
from qttable import *

#http://www.digitalfanatics.org/projects/qt_tutorial/chapter13.html
#Mirar de http://www.handhelds.org/~zecke/apidocs/libopie/html/ocheckitem_8cpp-source.html
#Mirar tablestatistics.py de ejemplos de PyQT

class ButtonTableItem(QTableItem):
	"""Esta clase hereda de QTableItem. Modifica dos aspectos de estos:
		Por un lado inhabilita el texto que muestra normalmente
		Por otro, utiliza un botón para que al ser pulsado lanza el dialogo de casos
		"""

	def __init__(self,tabla,dcasos):
		"""la inicialización llama al padre"""
		QTableItem.__init__(self,tabla,QTableItem.WhenCurrent,"...") 
		self.setReplaceable(False)
		self.__dcasos=dcasos

	#FUNCIONES PUBLICAS
		
	def createEditor(self):
		"""Crea un editor para interactuar con el contenido.
		En este caso es un botón, ya que permite asignar una acción al click de ratón"""
		boton=QButton(self.table().viewport(),"Ventana abierta") #El padre es viewport, es invisible
		QObject.connect(boton,SIGNAL("clicked()"),self.__editar)
		QObject.connect(boton,SIGNAL("clicked()"),self.table(), SLOT("doValueChanged()"))
		return boton

	def setContentFromEditor(self,widget):
		"""Cuando el contenido de widget cambia, qt llama a esta función.
		Esta desactivada ya que el botón que actua de editor no dispone de información para devolver"""
		pass

	#FUNCIONES PRIVADAS
	def __editar(self):
		"""Abre el dialogo de casos, pasando la columna como valor"""
		self.__dcasos.mostrar(self.row())

class WidgetLista:
	"""Clase base de los widgets que se gestionan una lista"""

	def __init__(self,lista=None):
		self.seleccion=None
		if lista:
			self.lista=lista
		else:
			self.lista=[]

	def aplicarlista(self):
		for variable in self.lista:
			self.insertItem(variable)

class ListBoxLista(QListBox,WidgetLista):
	"""ListBox que establece sus valores a partir de funciones auxiliares que trabajan sobre listas"""
	def __init__(self,parent,lista=None):
		WidgetLista.__init__(self,lista)
		QListBox.__init__(self,parent,"listBoxlista")


	def obtenerseleccion(self):
		"""Extrae la seleccion del ListBox"""
		pass

	def aplicarseleccion(self):
		"""Selecciona el elemento self.seleccion"""
		pass

	def myUpdate(self):
		self.obtenerseleccion()
		self.clear()
		self.aplicarlista()
		self.aplicarseleccion()

	def showEvent(self,ev):
		self.myUpdate()
		QListBox.showEvent(self,ev)



class ComboBoxLista(QComboBox,WidgetLista):
	"""QComboBox que establece sus valores a partir de funciones auxiliares que trabajan sobre listas"""
	#TODO: Falta conservar la seleccion con el show
	def __init__(self,parent):
		QComboBox.__init__(self,parent,"comboboxlista")
		WidgetLista.__init__(self)

	def myUpdate(self):
		self.clear()
		self.aplicarlista()

	def showEvent(self,ev):
		self.myUpdate()
		QComboBox.showEvent(self,ev)


class ComboBoxVars(ComboBoxLista):
	"""ComboBoxLista modificado que devuelve siempre una lista de variables actualizadas"""
	def __init__(self,parent,datos,factores=True):
		ComboBoxLista.__init__(self,parent)
		self.__datos=datos
		self.__factores=factores
		
	def aplicarlista(self):
		"""Redefine el show de QComboBox, añadiendo el listado de variables"""
		self.lista=self.__datos.arrTit(factores=self.__factores)
		ComboBoxLista.aplicarlista(self)



class ComboBoxFact(ComboBoxLista):
	"""ComboBox modificado que devuelve siempre una lista de variables actualizadas"""
	def __init__(self,parent,datos):
		self.__datos=datos
		ComboBoxLista.__init__(self,parent)
		
	def aplicarlista(self):
		"""Redefine el show de QComboBox, añadiendo el listado de variables"""
		self.lista=self.__datos.arrTit(tipo="Factor")
		ComboBoxLista.aplicarlista(self)



class ListBoxListaSimple(QVBox):
	def __init__(self,parent,nombre):
		QVBox.__init__(self,parent)
		self.label=QLabel(self)
		self.label.setText(nombre)
		self.caja=ListBoxLista(self)


class ListBoxListaDoble(QVBox):
	"""Dos ListBoxLista en disposición vertical"""
	def __init__(self,parent,nombre1,nombre2):
		QVBox.__init__(self,parent)
		self.label1=QLabel(self)
		self.label1.setText(nombre1)
		self.caja1=ListBoxLista(self)
		self.label2=QLabel(self)
		self.label2.setText(nombre2)
		self.caja2=ListBoxLista(self)

class ListBoxDobleComboBox(QVBox):
	"""Tiene un listbox y dos combobox"""
	def __init__(self,parent,nombre):
		QVBox.__init__(self,parent)
		self.label1=QLabel(self)
		self.label1.setText(nombre)
		self.caja=ListBoxLista(self)
		self.combo1=ComboBoxLista(self)
		self.combo2=ComboBoxLista(self)

class ListBoxDobleComboBoxListBoxListaDoble(QHBox):
	"""Bastante largo el nombre"""
	def __init__(self,parent,nombre1,nombre2,nombre3):
		QHBox.__init__(self,parent)
		self.zona1=ListBoxListaDoble(self,nombre1,nombre2)
		self.zona2=ListBoxDobleComboBox(self,nombre1)


class QGridButtonGroup(QButtonGroup):
	"""Reimplementación de QButtonGroup que organiza los botones de forma que ocupen lo menos posible
		Actualmente solo añade un QGridLayout de 2 columnas
	"""
	def __init__(self,parent,nombre):
		QButtonGroup.__init__(self,parent,"SeleccionMultiple")
		self.setGeometry(QRect(110,80,170,121))
		self.setColumnLayout(0,Qt.Vertical)
		self.layout().setSpacing(6)
		self.layout().setMargin(11)
		self.layout=QGridLayout(self.layout(),2)
		self.x=0
		self.y=0

	def insert(self,boton):
		QButtonGroup.insert(self,boton)
		self.layout.addWidget(boton,self.x,self.y)
		if self.y==0:
			self.y=1
		else:
			self.x+=1
			self.y=0

